<?php

namespace Drupal\media_entity_slideshow;

use Drupal\Component\Utility\Color;
use Drupal\Core\Image\ImageFactory;

/**
 * Generates a collage thumbnail.
 */
class ThumbnailGenerator {

  /**
   * The image factory service.
   *
   * @var \Drupal\Core\Image\ImageFactory
   */
  protected $imageFactory;

  /**
   * The height of the new thumbnail image.
   *
   * @var int
   */
  protected $height = 768;

  /**
   * The width of the new thumbnail image.
   *
   * @var int
   */
  protected $width = 1024;

  /**
   * Padding in the collage.
   *
   * @var int
   */
  protected $padding = 0;

  /**
   * The collage background color as HEX.
   *
   * @var string
   */
  protected $backgroundColor = '#FFFFFF';

  /**
   * ThumbnailGenerator constructor.
   *
   * @param \Drupal\Core\Image\ImageFactory $image_factory
   *   The image factory service.
   */
  public function __construct(ImageFactory $image_factory) {
    $this->imageFactory = $image_factory;
  }

  /**
   * Set the height of the new thumbnail image.
   *
   * @param int $height
   *   Height in px.
   *
   * @return \Drupal\media_entity_slideshow\ThumbnailGenerator
   *   This object.
   */
  public function setHeight($height) {
    $this->height = $height;
    return $this;
  }

  /**
   * Set the width of the new thumbnail image.
   *
   * @param int $width
   *   Width in px.
   *
   * @return \Drupal\media_entity_slideshow\ThumbnailGenerator
   *   This object.
   */
  public function setWidth($width) {
    $this->width = $width;
    return $this;
  }

  /**
   * Set the padding of the collage.
   *
   * @param int $padding
   *   Padding in px.
   *
   * @return \Drupal\media_entity_slideshow\ThumbnailGenerator
   *   This object.
   */
  public function setPadding($padding) {
    $this->padding = $padding;
    return $this;
  }

  /**
   * Set the background color.
   *
   * @param string $color
   *   The background color in HEX.
   *
   * @return \Drupal\media_entity_slideshow\ThumbnailGenerator
   *   This object.
   */
  public function setBackgroundColor($color) {
    $this->backgroundColor = $color;
    return $this;
  }

  /**
   * Create thumbnail for images.
   *
   * @param array $images
   *   Up to four image to create a collage from.
   *
   * @return \Drupal\Core\Image\Image|\Drupal\Core\Image\ImageInterface
   *   The new collage image.
   */
  public function create(array $images) {

    if (empty($images)) {
      throw new \RuntimeException('No images provided');
    }

    $loaded_images = [];
    foreach ($images as $image_src) {
      $loaded_images[] = $this->imageFactory->get($image_src);
    }

    $thumbnail = $this->imageFactory->get();
    $thumbnail->createNew($this->width, $this->height);

    $arr = Color::hexToRgb($this->backgroundColor);
    $color = imagecolorallocate($thumbnail->getToolkit()->getResource(), $arr['red'], $arr['green'], $arr['blue']);
    imagefill($thumbnail->getToolkit()->getResource(), 0, 0, $color);

    switch (count($loaded_images)) {
      case 1:
        /** @var \Drupal\Core\Image\ImageInterface $image */
        $image = current($loaded_images);
        $image->scaleAndCrop($this->width, $this->height);
        $thumbnail = $image;
        break;

      case 2:
        $loaded_images[0]->scaleAndCrop($this->width / 2 - $this->padding * 1.5, $this->height - $this->padding * 2);
        $resource = $loaded_images[0]->getToolkit()->getResource();
        imagecopy($thumbnail->getToolkit()->getResource(), $resource, $this->padding, $this->padding, 0, 0, imagesx($resource), imagesy($resource));
        imagedestroy($resource);

        $loaded_images[1]->scaleAndCrop($this->width / 2 - $this->padding * 1.5, $this->height - $this->padding * 2);
        $resource = $loaded_images[1]->getToolkit()->getResource();
        imagecopy($thumbnail->getToolkit()->getResource(), $resource, $this->width / 2 + $this->padding * 0.5, $this->padding, 0, 0, imagesx($resource), imagesy($resource));
        imagedestroy($resource);

        break;

      case 3:
        $loaded_images[0]->scaleAndCrop($this->width / 2 - $this->padding * 1.5, $this->height / 2 - $this->padding * 1.5);
        $resource = $loaded_images[0]->getToolkit()->getResource();
        imagecopy($thumbnail->getToolkit()->getResource(), $resource, $this->padding, $this->padding, 0, 0, imagesx($resource), imagesy($resource));
        imagedestroy($resource);

        $loaded_images[1]->scaleAndCrop($this->width / 2 - $this->padding * 1.5, $this->height / 2 - $this->padding * 1.5);
        $resource = $loaded_images[1]->getToolkit()->getResource();
        imagecopy($thumbnail->getToolkit()->getResource(), $resource, $this->width / 2 + $this->padding * 0.5, $this->padding, 0, 0, imagesx($resource), imagesy($resource));
        imagedestroy($resource);

        $loaded_images[2]->scaleAndCrop($this->width - $this->padding * 2, $this->height / 2 - $this->padding * 1.5);
        $resource = $loaded_images[2]->getToolkit()->getResource();
        imagecopy($thumbnail->getToolkit()->getResource(), $resource, $this->padding, $this->height / 2 + $this->padding * 0.5, 0, 0, imagesx($resource), imagesy($resource));
        imagedestroy($resource);

        break;

      default:
        $loaded_images[0]->scaleAndCrop($this->width / 2 - $this->padding * 1.5, $this->height / 2 - $this->padding * 1.5);
        $resource = $loaded_images[0]->getToolkit()->getResource();
        imagecopy($thumbnail->getToolkit()->getResource(), $resource, $this->padding, $this->padding, 0, 0, imagesx($resource), imagesy($resource));
        imagedestroy($resource);

        $loaded_images[1]->scaleAndCrop($this->width / 2 - $this->padding * 1.5, $this->height / 2 - $this->padding * 1.5);
        $resource = $loaded_images[1]->getToolkit()->getResource();
        imagecopy($thumbnail->getToolkit()->getResource(), $resource, $this->width / 2 + $this->padding * 0.5, $this->padding, 0, 0, imagesx($resource), imagesy($resource));
        imagedestroy($resource);

        $loaded_images[2]->scaleAndCrop($this->width / 2 - $this->padding * 1.5, $this->height / 2 - $this->padding * 1.5);
        $resource = $loaded_images[2]->getToolkit()->getResource();
        imagecopy($thumbnail->getToolkit()->getResource(), $resource, $this->padding, $this->height / 2 + $this->padding * 0.5, 0, 0, imagesx($resource), imagesy($resource));
        imagedestroy($resource);

        $loaded_images[3]->scaleAndCrop($this->width / 2 - $this->padding * 1.5, $this->height / 2 - $this->padding * 1.5);
        $resource = $loaded_images[3]->getToolkit()->getResource();
        imagecopy($thumbnail->getToolkit()->getResource(), $resource, $this->width / 2 + $this->padding * 0.5, $this->height / 2 + $this->padding * 0.5, 0, 0, imagesx($resource), imagesy($resource));
        imagedestroy($resource);

        break;
    }

    return $thumbnail;
  }

}
